import java.util.ArrayList;
import java.util.Collection;

import javassist.ClassPool;
import javassist.CtClass;
import javassist.bytecode.AnnotationsAttribute;
import javassist.bytecode.ConstPool;
import javassist.bytecode.annotation.Annotation;
import javassist.bytecode.annotation.StringMemberValue;

import javax.persistence.Table;

public class ReflectionSample {

    public static void main(final String[] args) {
        /*
         * It's no solution to provide just the class names as a string,
         * what will only work, if no application part accessed the classes already.
         */
        final Collection<Class<?>> classes = new ArrayList<Class<?>>();
        classes.add(ReflectionTest.class);
        doReflection(classes);

        final ReflectionTest test = new ReflectionTest();
        final Table table = test.getClass().getAnnotation(Table.class);

        System.out.println("name  : " + table.name());
        System.out.println("schema: " + table.schema());
    }

    private static void doReflection(final Collection<Class<?>> classes) {
        final ClassPool classPool = ClassPool.getDefault();
        for (final Class<?> clazz : classes) {
            try {
                final CtClass ctClass = classPool.get(clazz.getName());
                final ConstPool constPool = ctClass.getClassFile().getConstPool();
                final AnnotationsAttribute attribute = new AnnotationsAttribute(constPool, AnnotationsAttribute.visibleTag);
                final Table table = (Table) ctClass.getAnnotation(Table.class);
                final Annotation tableAnnotation = new Annotation("javax.persistence.Table", constPool);
                tableAnnotation.addMemberValue("name", new StringMemberValue(table.name(), constPool));
                tableAnnotation.addMemberValue("schema", new StringMemberValue("TestSchema", constPool));
                attribute.addAnnotation(tableAnnotation);
                ctClass.getClassFile().addAttribute(attribute);
                // This throws a java.lang.LinkageError ... attempted  duplicate class definition for name: "ReflectionTest"! 
                ctClass.toClass();
            } catch (Exception e) {
                e.printStackTrace();
            } 
        }
    }

}
